# Copyright 2014,  Jeff Buttars,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client.v30.action import Action
from acos_client.v30 import base
from acos_client.v30.partition import Partition


class System(base.BaseV30):

    @property
    def action(self):
        return Action(self.client)

    @property
    def partition(self):
        return Partition(self.client)

    def information(self):
        return self._get("/system")

    def stats(self, name='', max_retries=None, timeout=None, **kwargs):
        return self._get("/system/" + name + '/stats',
                         max_retries=max_retries, timeout=timeout,
                         **kwargs)
